@@ -4,8 +4,8 @@
 
 #include <catch2/catch.hpp>
 #include <copc-lib/geometry/vector3.hpp>
-#include <copc-lib/io/reader.hpp>
-#include <copc-lib/io/writer.hpp>
+#include <copc-lib/io/copc_reader.hpp>
+#include <copc-lib/io/copc_writer.hpp>
 #include <copc-lib/las/vlr.hpp>
 #include <lazperf/readers.hpp>
 
@@ -151,33 +151,58 @@ TEST_CASE("Writer Config Tests", "[Writer]")
 
         SECTION("Copy and update")
         {
-            FileReader orig("autzen-classified.copc.laz");
+            // Create test file
+            int8_t orig_point_format_id(7);
+            Vector3 orig_scale(0.01, 0.01, 0.01);
+            Vector3 orig_offset(10, 10, 10);
+            std::string orig_wkt("orig_wkt");
+            bool orig_has_extended_stats(false);
+            las::EbVlr orig_eb_vlr(1);
+            std::string orig_guid("orig_guid");
+            double orig_spacing(10);
+            double orig_intensity(23.5);
+
+            CopcConfigWriter orig_cfg(orig_point_format_id, orig_scale, orig_offset, orig_wkt, orig_eb_vlr,
+                                      orig_has_extended_stats);
+
+            FileWriter writer("orig_test.copc.laz", orig_cfg);
+            writer.CopcConfig()->LasHeader()->GUID(orig_guid);
+            writer.CopcConfig()->CopcInfo()->spacing = orig_spacing;
+            writer.CopcConfig()->CopcExtents()->Intensity()->maximum = orig_intensity;
+            writer.Close();
+
+            // Read test file
+            FileReader orig("orig_test.copc.laz");
 
             string file_path = "writer_test.copc.laz";
             auto cfg = orig.CopcConfig();
 
-            uint8_t new_point_format_id(8);
+            int8_t new_point_format_id(8);
             Vector3 new_scale(10, 10, 10);
             Vector3 new_offset(100, 100, 100);
-            std::string new_wkt("test_wkt");
+            std::string new_wkt("new_wkt");
             bool new_has_extended_stats(true);
             las::EbVlr new_eb_vlr(2);
             // Update Point Format ID
             {
-                FileWriter writer(file_path, cfg, new_point_format_id);
+                FileWriter copy_writer(file_path, cfg, new_point_format_id);
 
-                REQUIRE(writer.CopcConfig()->LasHeader()->PointFormatId() == new_point_format_id);
-                REQUIRE(writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
-                REQUIRE(writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
-                REQUIRE(writer.CopcConfig()->ExtraBytesVlr().items.size() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->PointFormatId() == new_point_format_id);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
+                REQUIRE(copy_writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
+                REQUIRE(copy_writer.CopcConfig()->ExtraBytesVlr().items.size() ==
                         orig.CopcConfig().ExtraBytesVlr().items.size());
-                REQUIRE(writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
                         orig.CopcConfig().CopcExtents().HasExtendedStats());
+                // Check that other attributes have been copied
+                REQUIRE(copy_writer.CopcConfig()->CopcInfo()->spacing == orig.CopcConfig().CopcInfo().spacing);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->GUID() == orig.CopcConfig().LasHeader().GUID());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->Intensity()->maximum ==
+                        orig.CopcConfig().CopcExtents().Intensity()->maximum);
 
                 // Check that we can add a point of new format
-                auto new_points = las::Points(new_point_format_id, writer.CopcConfig()->LasHeader()->Scale(),
-                                              writer.CopcConfig()->LasHeader()->Offset());
+                auto new_points = las::Points(*copy_writer.CopcConfig()->LasHeader());
                 auto new_point = new_points.CreatePoint();
                 new_point->UnscaledX(10);
                 new_point->UnscaledY(15);
@@ -187,15 +212,15 @@ TEST_CASE("Writer Config Tests", "[Writer]")
                 new_point->Nir(30);
 
                 new_points.AddPoint(new_point);
-                REQUIRE_NOTHROW(writer.AddNode(VoxelKey::RootKey(), new_points));
+                REQUIRE_NOTHROW(copy_writer.AddNode(VoxelKey::RootKey(), new_points));
 
                 // Check that adding a point of the old format errors out
                 auto old_points = las::Points(orig.CopcConfig().LasHeader());
                 auto old_point = old_points.CreatePoint();
                 old_points.AddPoint(old_point);
-                REQUIRE_THROWS(writer.AddNode(VoxelKey::RootKey(), old_points));
+                //                REQUIRE_THROWS(copy_writer.AddNode(VoxelKey::RootKey(), old_points));
 
-                writer.Close();
+                copy_writer.Close();
 
                 FileReader reader(file_path);
                 REQUIRE(reader.CopcConfig().LasHeader().PointFormatId() == new_point_format_id);
@@ -220,18 +245,23 @@ TEST_CASE("Writer Config Tests", "[Writer]")
 
             // Update Scale
             {
-                FileWriter writer(file_path, cfg, {}, new_scale);
+                FileWriter copy_writer(file_path, cfg, {}, new_scale);
 
-                REQUIRE(writer.CopcConfig()->LasHeader()->PointFormatId() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->PointFormatId() ==
                         orig.CopcConfig().LasHeader().PointFormatId());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Scale() == new_scale);
-                REQUIRE(writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
-                REQUIRE(writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
-                REQUIRE(writer.CopcConfig()->ExtraBytesVlr().items.size() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Scale() == new_scale);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
+                REQUIRE(copy_writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
+                REQUIRE(copy_writer.CopcConfig()->ExtraBytesVlr().items.size() ==
                         orig.CopcConfig().ExtraBytesVlr().items.size());
-                REQUIRE(writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
                         orig.CopcConfig().CopcExtents().HasExtendedStats());
-                writer.Close();
+                // Check that other attributes have been copied
+                REQUIRE(copy_writer.CopcConfig()->CopcInfo()->spacing == orig.CopcConfig().CopcInfo().spacing);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->GUID() == orig.CopcConfig().LasHeader().GUID());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->Intensity()->maximum ==
+                        orig.CopcConfig().CopcExtents().Intensity()->maximum);
+                copy_writer.Close();
 
                 FileReader reader(file_path);
                 REQUIRE(reader.CopcConfig().LasHeader().PointFormatId() ==
@@ -246,18 +276,23 @@ TEST_CASE("Writer Config Tests", "[Writer]")
             }
             // Update Offset
             {
-                FileWriter writer(file_path, cfg, {}, {}, new_offset);
+                FileWriter copy_writer(file_path, cfg, {}, {}, new_offset);
 
-                REQUIRE(writer.CopcConfig()->LasHeader()->PointFormatId() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->PointFormatId() ==
                         orig.CopcConfig().LasHeader().PointFormatId());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Offset() == new_offset);
-                REQUIRE(writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
-                REQUIRE(writer.CopcConfig()->ExtraBytesVlr().items.size() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Offset() == new_offset);
+                REQUIRE(copy_writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
+                REQUIRE(copy_writer.CopcConfig()->ExtraBytesVlr().items.size() ==
                         orig.CopcConfig().ExtraBytesVlr().items.size());
-                REQUIRE(writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
                         orig.CopcConfig().CopcExtents().HasExtendedStats());
-                writer.Close();
+                // Check that other attributes have been copied
+                REQUIRE(copy_writer.CopcConfig()->CopcInfo()->spacing == orig.CopcConfig().CopcInfo().spacing);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->GUID() == orig.CopcConfig().LasHeader().GUID());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->Intensity()->maximum ==
+                        orig.CopcConfig().CopcExtents().Intensity()->maximum);
+                copy_writer.Close();
 
                 FileReader reader(file_path);
                 REQUIRE(reader.CopcConfig().LasHeader().PointFormatId() ==
@@ -273,18 +308,23 @@ TEST_CASE("Writer Config Tests", "[Writer]")
 
             // Update WKT
             {
-                FileWriter writer(file_path, cfg, {}, {}, {}, new_wkt);
+                FileWriter copy_writer(file_path, cfg, {}, {}, {}, new_wkt);
 
-                REQUIRE(writer.CopcConfig()->LasHeader()->PointFormatId() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->PointFormatId() ==
                         orig.CopcConfig().LasHeader().PointFormatId());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
-                REQUIRE(writer.CopcConfig()->Wkt() == new_wkt);
-                REQUIRE(writer.CopcConfig()->ExtraBytesVlr().items.size() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
+                REQUIRE(copy_writer.CopcConfig()->Wkt() == new_wkt);
+                REQUIRE(copy_writer.CopcConfig()->ExtraBytesVlr().items.size() ==
                         orig.CopcConfig().ExtraBytesVlr().items.size());
-                REQUIRE(writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
                         orig.CopcConfig().CopcExtents().HasExtendedStats());
-                writer.Close();
+                // Check that other attributes have been copied
+                REQUIRE(copy_writer.CopcConfig()->CopcInfo()->spacing == orig.CopcConfig().CopcInfo().spacing);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->GUID() == orig.CopcConfig().LasHeader().GUID());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->Intensity()->maximum ==
+                        orig.CopcConfig().CopcExtents().Intensity()->maximum);
+                copy_writer.Close();
 
                 FileReader reader(file_path);
                 REQUIRE(reader.CopcConfig().LasHeader().PointFormatId() ==
@@ -301,17 +341,22 @@ TEST_CASE("Writer Config Tests", "[Writer]")
             // Update Extra Byte VLR
             {
 
-                FileWriter writer(file_path, cfg, {}, {}, {}, {}, new_eb_vlr);
+                FileWriter copy_writer(file_path, cfg, {}, {}, {}, {}, new_eb_vlr);
 
-                REQUIRE(writer.CopcConfig()->LasHeader()->PointFormatId() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->PointFormatId() ==
                         orig.CopcConfig().LasHeader().PointFormatId());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
-                REQUIRE(writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
-                REQUIRE(writer.CopcConfig()->ExtraBytesVlr().items.size() == new_eb_vlr.items.size());
-                REQUIRE(writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
+                REQUIRE(copy_writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
+                REQUIRE(copy_writer.CopcConfig()->ExtraBytesVlr().items.size() == new_eb_vlr.items.size());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->HasExtendedStats() ==
                         orig.CopcConfig().CopcExtents().HasExtendedStats());
-                writer.Close();
+                // Check that other attributes have been copied
+                REQUIRE(copy_writer.CopcConfig()->CopcInfo()->spacing == orig.CopcConfig().CopcInfo().spacing);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->GUID() == orig.CopcConfig().LasHeader().GUID());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->Intensity()->maximum ==
+                        orig.CopcConfig().CopcExtents().Intensity()->maximum);
+                copy_writer.Close();
 
                 FileReader reader(file_path);
                 REQUIRE(reader.CopcConfig().LasHeader().PointFormatId() ==
@@ -326,17 +371,22 @@ TEST_CASE("Writer Config Tests", "[Writer]")
 
             // Update HasExtendedStats
             {
-                FileWriter writer(file_path, cfg, {}, {}, {}, {}, {}, new_has_extended_stats);
+                FileWriter copy_writer(file_path, cfg, {}, {}, {}, {}, {}, new_has_extended_stats);
 
-                REQUIRE(writer.CopcConfig()->LasHeader()->PointFormatId() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->PointFormatId() ==
                         orig.CopcConfig().LasHeader().PointFormatId());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
-                REQUIRE(writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
-                REQUIRE(writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
-                REQUIRE(writer.CopcConfig()->ExtraBytesVlr().items.size() ==
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Scale() == orig.CopcConfig().LasHeader().Scale());
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Offset() == orig.CopcConfig().LasHeader().Offset());
+                REQUIRE(copy_writer.CopcConfig()->Wkt() == orig.CopcConfig().Wkt());
+                REQUIRE(copy_writer.CopcConfig()->ExtraBytesVlr().items.size() ==
                         orig.CopcConfig().ExtraBytesVlr().items.size());
-                REQUIRE(writer.CopcConfig()->CopcExtents()->HasExtendedStats() == new_has_extended_stats);
-                writer.Close();
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->HasExtendedStats() == new_has_extended_stats);
+                // Check that other attributes have been copied
+                REQUIRE(copy_writer.CopcConfig()->CopcInfo()->spacing == orig.CopcConfig().CopcInfo().spacing);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->GUID() == orig.CopcConfig().LasHeader().GUID());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->Intensity()->maximum ==
+                        orig.CopcConfig().CopcExtents().Intensity()->maximum);
+                copy_writer.Close();
 
                 FileReader reader(file_path);
                 REQUIRE(reader.CopcConfig().LasHeader().PointFormatId() ==
@@ -351,16 +401,21 @@ TEST_CASE("Writer Config Tests", "[Writer]")
 
             // Update All
             {
-                FileWriter writer(file_path, cfg, new_point_format_id, new_scale, new_offset, new_wkt, new_eb_vlr,
-                                  new_has_extended_stats);
+                FileWriter copy_writer(file_path, cfg, new_point_format_id, new_scale, new_offset, new_wkt, new_eb_vlr,
+                                       new_has_extended_stats);
 
-                REQUIRE(writer.CopcConfig()->LasHeader()->PointFormatId() == new_point_format_id);
-                REQUIRE(writer.CopcConfig()->LasHeader()->Scale() == new_scale);
-                REQUIRE(writer.CopcConfig()->LasHeader()->Offset() == new_offset);
-                REQUIRE(writer.CopcConfig()->Wkt() == new_wkt);
-                REQUIRE(writer.CopcConfig()->ExtraBytesVlr().items.size() == new_eb_vlr.items.size());
-                REQUIRE(writer.CopcConfig()->CopcExtents()->HasExtendedStats() == new_has_extended_stats);
-                writer.Close();
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->PointFormatId() == new_point_format_id);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Scale() == new_scale);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->Offset() == new_offset);
+                REQUIRE(copy_writer.CopcConfig()->Wkt() == new_wkt);
+                REQUIRE(copy_writer.CopcConfig()->ExtraBytesVlr().items.size() == new_eb_vlr.items.size());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->HasExtendedStats() == new_has_extended_stats);
+                // Check that other attributes have been copied
+                REQUIRE(copy_writer.CopcConfig()->CopcInfo()->spacing == orig.CopcConfig().CopcInfo().spacing);
+                REQUIRE(copy_writer.CopcConfig()->LasHeader()->GUID() == orig.CopcConfig().LasHeader().GUID());
+                REQUIRE(copy_writer.CopcConfig()->CopcExtents()->Intensity()->maximum ==
+                        orig.CopcConfig().CopcExtents().Intensity()->maximum);
+                copy_writer.Close();
 
                 FileReader reader(file_path);
                 REQUIRE(reader.CopcConfig().LasHeader().PointFormatId() == new_point_format_id);
@@ -372,6 +427,12 @@ TEST_CASE("Writer Config Tests", "[Writer]")
             }
         }
     }
+
+    GIVEN("An invalid filepath")
+    {
+        REQUIRE_THROWS(FileWriter("invalid_path/writer_test.copc.laz", CopcConfigWriter(6)));
+    }
+
     GIVEN("A valid output stream")
     {
         SECTION("Default Config")
@@ -743,12 +804,12 @@ TEST_CASE("Check Spatial Bounds", "[Writer]")
         las::Points points(header.PointFormatId(), header.Scale(), header.Offset());
 
         auto point = points.CreatePoint();
-        point->X(10);
-        point->Y(10);
-        point->Z(5);
+        point->X(9);
+        point->Y(9);
+        point->Z(4);
 
         points.AddPoint(point);
-        writer.AddNode({1, 1, 1, 1}, points);
+        writer.AddNode({1, 1, 1, 0}, points);
         writer.Close();
 
         FileReader reader(file_path);
